## Script: 		    svvmcdot.ps1
## Version: 	    0.3
## Author:          Dario Furigo <dario.furigo@itpoint.ch>
## Description: 	VSC 4.2.x does not support snapvault update with cDOT. This script supports vault relationships and also vault to mirror cascades.
##              	This script also takes care of renaming the snapshots from _recent to _DATETIME for vault destinations.
## History:		    0.1 - 17.10.2014 - First draft
##                  0.2 - 23.10.2014 - Bugfixes and snapshot rename
##                  0.3 - 24.10.2014 - Crypt and decrypt passwords, added vault to mirror cascading
##                  0.4 - 13.02.2015 - Added support for OnTap 8.3 (mirror-vault)


##########################
## INPUT
##########################

Param
(
    [Parameter(Mandatory=$False)][string]$logdir = "C:\temp\svvmcdot",
    [Parameter(Mandatory=$False)][string]$label = "VSC_DAILY",
    [Parameter(Mandatory=$False)][string]$user = "user",
    [Parameter(Mandatory=$False)][string]$password = "password",
    [Parameter(Mandatory=$False)][string]$cryptpw,
    [Parameter(Mandatory=$False)][switch]$docrypt
)


##########################
## MODUL
##########################

Import-Module DataOnTap


##########################
## VARIABLES
##########################

$configfile = "$PSScriptRoot"+"\svvmcdot.conf"
$logfile = "$logdir"+"\svvmcdot_"+(get-date -UFormat "%y%m%d")+".log"
$now = get-date -Format "dd-MM-yyyy HH:mm:ss"
$scriptversion = get-date -Format "ddMMyyyyHHmmss"


##########################
## FUNCTIONS
##########################

# The getSVMconfig function searches for the svm entry in the configfile which is in the same directory as the script
# and returns the name and ip addresses of the searched SVMs
Function getSVMconfig($searchParam)
{
    $result = Select-String -Path $configfile -Pattern "$searchParam"

    if ( ($result | Measure-Object -Line).Lines -eq 1)
    {
        $tmpSVM = new-object psobject
        Add-Member -InputObject $tmpSVM -MemberType NoteProperty -Name Svmname -Value ($result.line.Split(",")[0])
        Add-Member -InputObject $tmpSVM -MemberType NoteProperty -Name MgmtIP -Value ($result.line.Split(",")[1])
        Add-Member -InputObject $tmpSVM -MemberType NoteProperty -Name NfsIP -Value ($result.line.Split(",")[2])
        return $tmpSVM
    } else {
        return ""
    }
}

Function encryptString($string)
{
    $r = new-Object System.Security.Cryptography.RijndaelManaged  # use Rijndael symmetric key encryption
    $c = $r.CreateEncryptor((1..16), (1..16))    # Set the key and initialisation vector to 128-bytes each of (1..16)
    $ms = new-Object IO.MemoryStream
    $cs = new-Object Security.Cryptography.CryptoStream $ms,$c,"Write" # Target data stream, transformation, and mode
    $sw = new-Object IO.StreamWriter $cs
    $sw.Write($String)       # Write the string through the crypto stream into the memory stream
    $sw.Close()
    $cs.Close()
    $ms.Close()
    $r.Clear()
    [byte[]]$result = $ms.ToArray()      # Byte array from the encrypted memory stream
    $encstring = [Convert]::ToBase64String($result)    # Convert to base64 for transport

    return $encstring
}

Function decryptString($encstring)
{
    $Encrypted = [Convert]::FromBase64String($encstring)   # Convert the encrypted string to a byte array
    $r = new-Object System.Security.Cryptography.RijndaelManaged  # use Rijndael symmetric key encryption
    $d = $r.CreateDecryptor((1..16), (1..16))    # Set the key and initialisation vector to 128-bytes each of (1..16)

    $ms = new-Object IO.MemoryStream @(,$Encrypted)    # Create a memorystream from a single-element name/value pair hash table of the byte array
    $cs = new-Object Security.Cryptography.CryptoStream $ms,$d,"Read" # Target data stream, transformation, and mode
    $sr = new-Object IO.StreamReader $cs     # Read the string through the crypto stream from the encrypted memory stream
    $string = $sr.ReadToEnd()      # Write the unencrypted string
    $sr.Close()
    $cs.Close()
    $ms.Close()
    $r.Clear()

    return $string
}


##########################
## MAIN
##########################

if ( $docrypt )
{
    $string = Read-Host -Prompt "Password"
    Write-Output "Crypted password is: "(encryptString($string))
    exit(0)
}

# If backup phase is "POST" and not "PRE"
if ((Get-ChildItem Env:BACKUP_PHASE).Value -eq "POST_BACKUP" ) 
{

    # Check and create logdir
    if ( (Test-Path -Path $logdir -PathType Container) -eq $false )
    {
        New-Item -Path $logdir -ItemType Directory -Force | Out-Null
    }

    Add-Content -Path $logfile -Value ""
    Add-Content -Path $logfile -Value "-------------------------------------------------"
    Add-Content -Path $logfile -Value "Started Script version $scriptversion at $now"
    Add-Content -path $logfile -Value "Param user: $user"
    Add-Content -path $logfile -Value "Param password is hidden"
    Add-Content -path $logfile -Value "Param snapmirror-label: $label"
    Add-Content -Path $logfile -Value ""

    # Check for configfile
    if ( (Test-Path -Path $configfile -PathType Leaf) -eq $false )
    {
        Add-Content -path $logfile -value "Config file does not exist!"
        Add-Content -path $logfile -value "Please create the config $configfile with the following content:"
        Add-Content -path $logfile -value "<SVM,MGMTIP,NFSIP> like"
        Add-Content -path $logfile -value "SVM002,10.2.5.22,10.0.0.50`n"
        exit(1)
    }

    # Process password
    if ( $cryptpw )
    {
        $password = decryptString($cryptpw)
    } else {
        Add-Content -Path $logfile -Value "Please encrypt your password!!"
    }
    $pass = ConvertTo-SecureString -String $password -AsPlainText -Force
    $cred = New-Object system.Management.Automation.PSCredential -ArgumentList $user,$pass


    Get-ChildItem Env: | ? {$_.Name -like "STORAGE_SNAPSHOT.*"} | ForEach-Object {
        
        $SnapshotValues = $_.Value.Split(":")
        $Svmnfsip = $SnapshotValues[0]
        $Volume = $SnapshotValues[1].Split("/")[2]
        $Snapshot = $SnapshotValues[2]
        $NewSnapshot = ($Snapshot.TrimEnd("recent"))+(Get-ChildItem Env:BACKUP_DATE).Value+(Get-ChildItem Env:BACKUP_TIME).Value

        Add-Content -path $logfile -value "Searching for SVM with NFS IP $Svmnfsip..."
        $SVMConfig = getSVMConfig($Svmnfsip)

        if ($SVMConfig -eq "")
        {
            Add-Content -path $logfile -value "ERROR: There are multiple or no configfile entry for svm $Svmnfsip!!"
        } else {
            Add-Content -path $logfile -value ("Connecting to "+$SVMConfig.Svmname+" ("+$SVMConfig.MgmtIP+")...")

            $conn = Connect-NcController -name $SVMConfig.MgmtIP -HTTPS -Credential $cred

            if ($conn -eq $null)
            {
		        Add-Content -path $logfile -value ("ERROR: Connection to SVM "+$SVMConfig.Svmname+" failed!!")    
            } else {

                Add-Content -path $logfile -value "Setting label $label to $Volume..."
                Set-NcSnapshot -Volume $Volume -Snapshot $Snapshot -SnapmirrorLabel $label -Vserver $SVMConfig.Svmname | Out-Null

                Add-Content -path $logfile -value ("Searching for vault destinations for Volume "+$Volume+"...")
                Get-NcSnapmirrorDestination -SourceVolume $Volume -SourceVserver $SVMConfig.Svmname | ? {$_.RelationshipType -eq "vault" -or ($_.RelationshipType -eq "extended_data_protection" -and $_.PolicyType -eq "mirror_vault")} | ForEach-Object {
                    $VAdestSVM = $_.DestinationVserver
                    $VAdestVolume = $_.DestinationVolume
                    $VAdestLocation = $_.DestinationLocation
                    Add-Content -path $logfile -value ("Found destination "+$VAdestLocation+"...")


                    Add-Content -path $logfile -value ("Searching for Backup SVM with Name "+$VAdestSVM+"...")
                    $BackupSVMConfig = getSVMConfig($VAdestSVM)

                    if ($BackupSVMConfig -eq "")
                    {
                        Add-Content -path $logfile -value ("ERROR: There are multiple or no configfile entry for svm "+$VAdestSVM+"!!")
                    } else {
                        Add-Content -path $logfile -value ("Connecting to "+$BackupSVMConfig.Svmname+" ("+$BackupSVMConfig.MgmtIP+")...")

                        $conn = Connect-NcController -name $BackupSVMConfig.MgmtIP -HTTPS -Credential $cred

                        if ($conn -eq $null)
                        {
		                    Add-Content -path $logfile -value ("ERROR: Connection to Backup SVM "+$BackupSVMConfig.Svmname+" failed!!")    
                        } else {

                            Add-Content -path $logfile -value ("Doing a snapmirror update on "+$VAdestLocation+"...")
                            $VAUPresult = Invoke-NcSnapmirrorUpdate -SourceVserver $SVMConfig.Svmname -SourceVolume $Volume -DestinationVserver $BackupSVMConfig.Svmname -DestinationVolume $VAdestVolume
                            if ($VAUPresult.Status -eq "succeeded")
                            {
                                Add-Content -path $logfile -value ("SnapMirror update for "+$VAdestLocation+" was successful...")

                                while ( (get-ncsnapmirror -SourceVserver $SVMConfig.Svmname -SourceVolume $Volume -DestinationVserver $BackupSVMConfig.Svmname -DestinationVolume $VAdestVolume).Status -ne "idle" ) 
                                { 
	                                Start-Sleep 20
                                }

                                Add-Content -path $logfile -value ("Rename Snapshot "+$Snapshot+" to "+$NewSnapshot+" for Volume "+$VAdestVolume+"...")
                                Rename-NcSnapshot -Volume $VAdestVolume -Snapshot $Snapshot -NewName $NewSnapshot
                                #if ( ! (Rename-NcSnapshot -Volume $VAdestVolume -Snapshot $Snapshot -NewName $NewSnapshot))
                                #{
                                #    Add-Content -path $logfile -value ("Snapshot rename failed!!")
                                #}

                                Add-Content -path $logfile -value ("Searching for data_protection destinations for Volume "+$VAdestVolume+"...")
                                Get-NcSnapmirrorDestination -SourceVolume $VAdestVolume -SourceVserver $BackupSVMConfig.Svmname | ? {$_.RelationshipType -eq "data_protection"} | ForEach-Object {  
                                    $MIdestSVM = $_.DestinationVserver
                                    $MIdestVolume = $_.DestinationVolume
                                    $MIdestLocation = $_.DestinationLocation
                                    Add-Content -path $logfile -value ("Found destination "+$MIdestLocation+"...")


                                    Add-Content -path $logfile -value ("Searching for Backup SVM with Name "+$MIdestSVM+"...")
                                    $MirrorSVMConfig = getSVMConfig($MIdestSVM)

                                    if ($MirrorSVMConfig -eq "")
                                    {
                                        Add-Content -path $logfile -value ("ERROR: There are multiple or no configfile entry for svm "+$MIdestSVM+"!!")
                                    } else {
                                        Add-Content -path $logfile -value ("Connecting to "+$MirrorSVMConfig.Svmname+" ("+$MirrorSVMConfig.MgmtIP+")...")

                                        $conn = Connect-NcController -name $MirrorSVMConfig.MgmtIP -HTTPS -Credential $cred

                                        if ($conn -eq $null)
                                        {
		                                    Add-Content -path $logfile -value ("ERROR: Connection to Backup SVM "+$MirrorSVMConfig.Svmname+" failed!!")    
                                        } else {

                                            Add-Content -path $logfile -value ("Doing a snapmirror update on "+$MIdestLocation+"...")
                                            $MIUPresult = Invoke-NcSnapmirrorUpdate -SourceVserver $BackupSVMConfig.Svmname -SourceVolume $VAdestVolume -DestinationVserver $MirrorSVMConfig.Svmname -DestinationVolume $MIdestVolume
                                            if ($MIUPresult.Status -eq "succeeded")
                                            {
                                                Add-Content -path $logfile -value ("SnapMirror update for "+$MIdestLocation+" was successful...")
                                            } else {
                                                Add-Content -path $logfile -value ("ERROR: SnapMirror update for "+$MIdestLocation+" was not successful: "+$MIUPresult.ErrorMessage+"...")
                                            }
                                        }
                                    }
                                }

                            } else {
                                Add-Content -path $logfile -value ("ERROR: SnapMirror update for "+$VAdestLocation+" was not successful: "+$VAUPresult.ErrorMessage+"...")
                            }
                        }                    
                    }

                }
            }
        }
        
    }
 
    $now2 = get-date -Format "dd-MM-yyyy HH:mm:ss"
    Add-Content -Path $logfile -Value ""
    Add-Content -Path $logfile -Value "Ended script (version $scriptversion) at $now2"
    Add-Content -Path $logfile -Value "-------------------------------------------------"
    Add-Content -Path $logfile -Value ""
    exit(0)

}

exit(0)
